use crate::ui::UIWidget;
use maud::{Markup, Render, html};

#[allow(non_snake_case)]
pub fn Flex<T: UIWidget + 'static>(inner: T) -> FlexWidget {
    FlexWidget(Box::new(inner), vec![], false)
}

pub enum Justify {
    Center,
    Between,
}

pub struct FlexWidget(Box<dyn UIWidget>, Vec<String>, bool);

impl Render for FlexWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl FlexWidget {
    #[must_use]
    pub fn full_center(mut self) -> Self {
        self.1.push("items-center".to_owned());
        self.1.push("justify-center".to_owned());
        self
    }

    #[must_use]
    pub const fn group(mut self) -> Self {
        self.2 = true;
        self
    }

    #[must_use]
    pub fn justify(mut self, value: Justify) -> Self {
        let class = match value {
            Justify::Center => "justify-center".to_owned(),
            Justify::Between => "justify-between".to_owned(),
        };

        self.1.push(class);
        self
    }

    #[must_use]
    pub fn items_center(mut self) -> Self {
        self.1.push("items-center".to_owned());
        self
    }

    #[must_use]
    pub fn gap(mut self, amount: u32) -> Self {
        self.1.push(format!("gap-{amount}"));
        self
    }
}

impl UIWidget for FlexWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        let mut res = vec!["flex".to_string()];
        res.extend_from_slice(&self.1);
        res
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() && !self.2 {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
